SystemOrganization addCategory: #LostInMorvanChat!
SystemOrganization addCategory: 'LostInMorvanChat-Client'!
SystemOrganization addCategory: 'LostInMorvanChat-Message'!
SystemOrganization addCategory: 'LostInMorvanChat-Server'!
SystemOrganization addCategory: 'LostInMorvanChat-User'!

ComposableModel subclass: #ChatConsole
	instanceVariableNames: 'list input chatClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatConsole commentStamp: 'FrancoisJarriges 6/14/2016 15:30' prior: 0!
I represent the console window to be used for the chat application to write and read messages.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	chatClient:		<ChatClient>
	input:			<TextInputFieldModel>
	list:				<ChatConsoleListModel>


    Implementation Points!

!ChatConsole class methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 14:16'!
attach: aChatClient

	| aChatConsole |
	
	aChatConsole := self new chatClient: aChatClient.
	aChatConsole openWithSpec whenClosedDo: [ aChatClient disconnect ].
	^ aChatConsole! !

!ChatConsole class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 11:53'!
defaultSpec
	<spec: #default>
	
	^ SpecLayout composed
			newColumn: [ :c |
				c add: #list; add: #input height: 30 ]; yourself
! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/14/2016 14:16'!
chatClient: aChatClient
	chatClient := aChatClient! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 14:16'!
initializeWidgets

	list := ChatConsoleListModel new.
	input := TextInputFieldModel new
		ghostText: 'Type your message here...';
		enabled: true;
		acceptBlock: [ :inputString |
			inputString = '>?' 
				ifTrue: [ chatClient getConnectedUsers ]
				ifFalse: [ chatClient send: inputString ].
			input text: '' ].
	self focusOrder add: input
! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:31'!
input
	^ input! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:31'!
list
	^ list! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 09:45'!
print: aCollectionOfChatMessages

	list addItems: aCollectionOfChatMessages 
! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 14:16'!
title
	^ chatClient login, ' on ', chatClient url! !

ListModel subclass: #ChatConsoleListModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatConsoleListModel commentStamp: 'FrancoisJarriges 6/2/2016 15:06' prior: 0!
I specialize the standard UI ListModel class for a chat console window.


    Implementation Points!

!ChatConsoleListModel methodsFor: 'api' stamp: 'FrancoisJarriges 6/2/2016 12:55'!
addItems: aList
	"Add a new list of items to the existing list."

	| oldContents |
	oldContents := listHolder value.
	listHolder value: {oldContents . (aList sorted: sortingBlockHolder value)} concatenation.
	listAnnouncer announce: (ValueChanged oldValue: oldContents newValue: listHolder value).
	! !

Announcement subclass: #ChatMessageAnnouncement
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatMessageAnnouncement commentStamp: 'FrancoisJarriges 6/3/2016 19:41' prior: 0!
I represent a new message  event  raised from a server thread at each reception of a new message post.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	message:		<ChatMessage>


    Implementation Points!

!ChatMessageAnnouncement class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/3/2016 19:54'!
for: aChatMessage
	
	 ^ self new message: aChatMessage! !

!ChatMessageAnnouncement methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/3/2016 20:34'!
message

	^ message! !

!ChatMessageAnnouncement methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/3/2016 19:55'!
message: aChatMessage

	message := aChatMessage! !

Object subclass: #ChatClient
	instanceVariableNames: 'url login exit messages console lastMessageIndex lastNetworkErrorString getConnectedUsers getLastMessageIndex getMessagesFromLastIndex putLogin putLogout putMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClient commentStamp: 'FrancoisJarriges 6/15/2016 10:57' prior: 0!
I am a chat client. I'm sending my own messages  to an HTTP server and getting all the messages sent by all the chat clients including me to display on a console. 

ChatClient connect: 'localhost' port: 8080 login: 'toto'.
ChatClient connect: 'eph-30278de7.swarm.pharocloud.com' login: 'toto'
 
Internal Representation and Key Implementation Points.

    Instance Variables
	url:							<ByteString>
	login:						<ByteString>
	exit:							<Boolean>
	messages:					<OrderedCollection>
	console:						<ChatConsole>
	lastMessageIndex:			<Integer>
	lastNetworkErrorString:		<ByteString>
	getConnectedUsers:			<ChatClientGetConnectedUsers>
	getLastMessageIndex:			<ChatClientGetLastMessageIndex>
	getMessagesFromLastIndex:  	<ChatClientGetMessagesFromLastIndex>
	postMessage:				<ChatClientPostMessage>
	postLogIn:					<ChatClientPostLogin>
	postLogout: 					<ChatClientPostLogout>
	

    Implementation Points!

!ChatClient class methodsFor: 'public' stamp: 'FrancoisJarriges 6/15/2016 09:09'!
connect: aHost login: aLogin

	^ self new
		host: aHost login: aLogin;
		start
! !

!ChatClient class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 11:19'!
connect: aHost port: aPort login: aLogin

	^ self new
		host: aHost port: aPort login: aLogin;
		start
! !

!ChatClient methodsFor: 'api' stamp: 'FrancoisJarriges 6/15/2016 14:11'!
disconnect
	"Here sending logout message is delegated to a light process in order to free the console immediately."

	exit := true .
	[ 
		putLogout basicRequest.
		
	] fork


! !

!ChatClient methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 20:26'!
getConnectedUsers

	
	console print:(((STONReader on: getConnectedUsers basicRequest readStream) next)
		collect: [ :eachUserLogin | ChatMessage 
			from: '-----' 
			text: eachUserLogin ])

! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/15/2016 08:33'!
getNewMessages
	"Gets the new messages that have been posted since the last request."
	"Prints the new messages on the console."

	| receivedChatMessages |
	
	receivedChatMessages := (STONReader on: getMessagesFromLastIndex basicRequest readStream) next.
	console print: (receivedChatMessages select: [ :eachChatMessage | 
		eachChatMessage sender ~= login ]).
	messages addAll: receivedChatMessages.
	^ receivedChatMessages size
	
! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/15/2016 09:09'!
host: aHost login: aLogin

	url := 'http://' , aHost.
	login := aLogin

! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 11:22'!
host: aHost port: aPort login: aLogin

	url := 'http://' , aHost , ':' , aPort asString.
	login := aLogin

! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/15/2016 14:11'!
initialize

	super initialize.
	exit := false.
	lastNetworkErrorString := ''.
	lastMessageIndex := 0.
	getConnectedUsers := ChatClientGetConnectedUsers from: self.
	getLastMessageIndex := ChatClientGetLastMessageIndex from: self.
	getMessagesFromLastIndex := ChatClientGetMessagesFromLastIndex from: self.
	putMessage := ChatClientPutMessage from: self.
	putLogin := ChatClientPutLogin from: self.
	putLogout :=  ChatClientPutLogout from: self.
	messages := OrderedCollection new! !

!ChatClient methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/14/2016 13:02'!
lastMessageIndex
	^ lastMessageIndex ! !

!ChatClient methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/2/2016 17:04'!
login
	^ login! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 11:11'!
printConnectionErrorMessage: aNetworkErrorString

	| errorMessage |
	
	aNetworkErrorString = lastNetworkErrorString ifFalse: [ 
		lastNetworkErrorString := aNetworkErrorString.
		errorMessage := ChatMessage 
			from: '-----' 
			text: lastNetworkErrorString.
		console print: errorMessage asOrderedCollection ]


	! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/15/2016 17:00'!
refreshMessages
	"Create and schedule a light process to get new posted messages every 2 seconds."
	"At client disconnection the process ends."
	
	[ 
		[ exit ] whileFalse: [
			lastMessageIndex := lastMessageIndex + self getNewMessages.
			(Delay forSeconds: 2) wait ] 
	] fork! !

!ChatClient methodsFor: 'api' stamp: 'FrancoisJarriges 6/15/2016 14:11'!
send: aString
	"Get the string input from the console and print it back to it."
	"Post a message to the server which text is this string."
	
	| newChatMessage |
	
	newChatMessage := ChatMessage from: login text: aString.
	console print: (OrderedCollection with: newChatMessage).
	putMessage basicRequest: newChatMessage 
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/15/2016 17:28'!
start

	console := ChatConsole attach: self.
	lastMessageIndex := (STONReader on: getLastMessageIndex basicRequest readStream) next.
	putLogin basicRequest.
	self refreshMessages! !

!ChatClient methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/2/2016 17:04'!
url
	^ url! !

Object subclass: #ChatClientHttpRequest
	instanceVariableNames: 'command userId chatClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

ChatClientHttpRequest class
	instanceVariableNames: 'path'!

!ChatClientHttpRequest commentStamp: 'FrancoisJarriges 6/14/2016 16:18' prior: 0!
I represent an abstract class of all HTTP requests sent by this  client chat application.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	chatClient:		<ChatClient>
	command:		<ByteString>
	userId:			<ByteString>
				
    Class Instance Variables 
	path:			<ByteString>


    Implementation Points!

ChatClientHttpRequest class
	instanceVariableNames: 'path'!

ChatClientHttpRequest subclass: #ChatClientGet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientGet commentStamp: 'FrancoisJarriges 6/14/2016 15:19' prior: 0!
I represent an abstract class of all HTTP GET requests sent by this  chat client.

Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientGet methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 14:22'!
basicRequest
		
	| responseString |
	
	[ responseString := ZnClient new 
		url: self command; 
		headerAt: 'ChatUserId' put: self userId;
		get ] 
		on: NetworkError 
		do: [ :thisNetworkError | 
			responseString := self handleNetWorkError: thisNetworkError ].
	^ responseString ! !

!ChatClientGet methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 14:28'!
handleNetWorkError: aNetworkError
	
	chatClient printConnectionErrorMessage: aNetworkError asString.
	^ STON toString: OrderedCollection new! !

ChatClientGet subclass: #ChatClientGetConnectedUsers
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientGetConnectedUsers commentStamp: 'FrancoisJarriges 6/14/2016 15:19' prior: 0!
I represent an HTTP request sent by this chat client to get all connected users from the server 
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientGetConnectedUsers class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/14/2016 16:18'!
initialize

	path := '/users/connected'! !

ChatClientGetConnectedUsers initialize!

ChatClientGet subclass: #ChatClientGetLastMessageIndex
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientGetLastMessageIndex commentStamp: 'FrancoisJarriges 6/14/2016 15:19' prior: 0!
I represent an HTTP request sent by this chat client to get the last message index from the server.
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientGetLastMessageIndex class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/14/2016 16:19'!
initialize
	
		path := '/messages/count'! !

ChatClientGetLastMessageIndex initialize!

!ChatClientGetLastMessageIndex methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 14:27'!
handleNetWorkError: aNetworkError
	
	chatClient printConnectionErrorMessage: aNetworkError asString.
	^ STON toString: chatClient lastMessageIndex ! !

ChatClientGet subclass: #ChatClientGetMessagesFromLastIndex
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientGetMessagesFromLastIndex commentStamp: 'FrancoisJarriges 6/14/2016 15:19' prior: 0!
I represent an HTTP  request sent by this chat client to get  all  the last messages posted to the server.
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientGetMessagesFromLastIndex class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/15/2016 16:30'!
initialize
	
	path := '/messages/from/'! !

ChatClientGetMessagesFromLastIndex initialize!

!ChatClientGetMessagesFromLastIndex methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 16:39'!
command

	^ super command, chatClient lastMessageIndex asString! !

!ChatClientHttpRequest class methodsFor: 'instance creation' stamp: 'FrancoisJarriges 6/14/2016 16:29'!
from: aChatClient

	path ifNil: [ self initialize ].
	^ self new chatClient: aChatClient; yourself! !

!ChatClientHttpRequest class methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 16:17'!
path

	^ path! !

!ChatClientHttpRequest methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 13:14'!
basicRequest

	^ self subclassResponsibility ! !

!ChatClientHttpRequest methodsFor: 'api' stamp: 'FrancoisJarriges 6/14/2016 13:41'!
basicRequest: anObject

	^ self subclassResponsibility ! !

!ChatClientHttpRequest methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 14:15'!
chatClient: aChatClient

	chatClient := aChatClient 
	! !

!ChatClientHttpRequest methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 16:19'!
command

	^ command ifNil: [ command := '{1}{2}' 
			format: { chatClient url . self class path } ]! !

!ChatClientHttpRequest methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 14:05'!
handleNetWorkError: aNetworkError

	^ self subclassResponsibility ! !

!ChatClientHttpRequest methodsFor: 'private' stamp: 'FrancoisJarriges 6/14/2016 14:19'!
userId

	^ userId ifNil: [ userId := chatClient login asCamelCase uncapitalized ]
	! !

ChatClientHttpRequest subclass: #ChatClientPut
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientPut commentStamp: 'FrancoisJarriges 6/15/2016 14:09' prior: 0!
I represent an abstract class of all HTTP PUT requests sent by this  chat client.

 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientPut methodsFor: 'api' stamp: 'FrancoisJarriges 6/15/2016 15:33'!
basicRequest

	[ ZnClient new
		url: self command;
		headerAt: 'ChatUserId' put: self userId;
		entity: (ZnEntity 
      			with: (STON toString: (ChatMessage
					from: chatClient login
					text: ''))
      			type: ZnMimeType applicationSton);
		put ]
		on: NetworkError 
		do: [ :thisNetworkError | 
			self handleNetWorkError: thisNetworkError ]! !

!ChatClientPut methodsFor: 'private' stamp: 'FrancoisJarriges 6/15/2016 07:50'!
handleNetWorkError: aNetworkError
	
	chatClient printConnectionErrorMessage: aNetworkError asString! !

ChatClientPut subclass: #ChatClientPutLogin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientPutLogin commentStamp: 'FrancoisJarriges 6/15/2016 14:05' prior: 0!
I represent an HTTP  request sent by this chat client to signal user login to the server.

 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientPutLogin class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/15/2016 10:55'!
initialize
	
	path := '/users/login'! !

ChatClientPutLogin initialize!

ChatClientPut subclass: #ChatClientPutLogout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientPutLogout commentStamp: 'FrancoisJarriges 6/15/2016 14:05' prior: 0!
I represent an HTTP  request sent by this chat client to signal user logout to the server.
 
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientPutLogout class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/15/2016 07:55'!
initialize
	
	path := '/users/logout'! !

ChatClientPutLogout initialize!

ChatClientPut subclass: #ChatClientPutMessage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClientPutMessage commentStamp: 'FrancoisJarriges 6/15/2016 14:09' prior: 0!
I represent an HTTP  request sent by this chat client to push  a message to the server.
 
Internal Representation and Key Implementation Points.


    Implementation Points!

!ChatClientPutMessage class methodsFor: 'class initialization' stamp: 'FrancoisJarriges 6/14/2016 16:19'!
initialize
	
	path := '/messages/add'! !

ChatClientPutMessage initialize!

!ChatClientPutMessage methodsFor: 'api' stamp: 'FrancoisJarriges 6/15/2016 14:03'!
basicRequest: aChatMessage

	[ ZnClient new
		url: self command;
		headerAt: 'ChatUserId' put: self userId;
		entity: (ZnEntity 
              with: (STON toString: aChatMessage)
              type: ZnMimeType applicationSton);
		put ]
		on: NetworkError 
		do: [ :thisNetworkError | 
			self handleNetWorkError: thisNetworkError ]! !

Object subclass: #ChatMessage
	instanceVariableNames: 'sender text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Message'!

!ChatMessage commentStamp: 'FrancoisJarriges 6/2/2016 14:57' prior: 0!
I'm a simple chat message. I identify my sender and my text.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	sender:		<Object>
	text:		<Object>


    Implementation Points!

!ChatMessage class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/1/2016 08:59'!
from: aSender text: aText

	^ self new sender: aSender; text: aText; yourself
! !

!ChatMessage class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 12:54'!
separator
	^ '>'! !

!ChatMessage methodsFor: 'printing' stamp: 'FrancoisJarriges 6/1/2016 15:01'!
printOn: aStream

	aStream 
		<< self sender; 
		<< self class separator;
		<< String space;
		<< self text; 
		<< String crlf! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
sender
	^ sender! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 08:10'!
sender: aByteString
	sender := aByteString! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
text
	^ text! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 08:10'!
text: aByteString
	text := aByteString! !

Object subclass: #ChatMessagesQueue
	instanceVariableNames: 'messages announcer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatMessagesQueue commentStamp: 'FrancoisJarriges 6/3/2016 19:46' prior: 0!
I represent a list of all the messages sent by clients in this chat application.

    Instance Variables
	messages:		<OrderedCollection>
	announcer: 		<Announcer>


    Implementation Points!

!ChatMessagesQueue methodsFor: 'adding' stamp: 'FrancoisJarriges 6/4/2016 03:59'!
add: aChatMessage
	messages add: aChatMessage 
! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/3/2016 19:46'!
announcer
	
	^ announcer ifNil: [ announcer := Announcer new ]! !

!ChatMessagesQueue methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 04:33'!
initialize

	super initialize.
	messages := OrderedCollection new.
	self announcer 
		when: ChatMessageAnnouncement 
		do: [ :thisAnnouncement |
			self add: thisAnnouncement message ]
 ! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/15/2016 15:23'!
listFrom: anIndex

	^ (anIndex between: 0 and: messages size - 1)
		ifTrue: [ messages copyFrom: anIndex + 1 to: messages size ]
		ifFalse: [ OrderedCollection new ]! !

!ChatMessagesQueue methodsFor: 'resetting' stamp: 'FrancoisJarriges 6/4/2016 04:13'!
reset
	self announcer unsubscribe: self.
	messages removeAll! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 09:12'!
size
	^ messages size! !

Object subclass: #ChatServer
	instanceVariableNames: 'teapotServer messagesQueue log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatServer commentStamp: 'FrancoisJarriges 6/14/2016 15:38' prior: 0!
I'm a chat server. I'm listening to HTTPRequests and managing a list of messages. 

My clients often request the messages I hold. Indeed, it may happen that client sends a lot of message and another one should request the difference between the messages it knows and the ones published on the server to be able to refresh correctly. 

I register some routes 
	/messages/ aNumber  get all the messages from aNumber to the end.
	/messages/count to get the current number of messages
	/messages/ add to add a message

ChatServer stopAll.
ChatServer startOn: 8080.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	teapotServer:		<Teapot>
	messagesQueue:		<ChatMessagesQueue>
	log					<ChatUsersLogged>


    Implementation Points
	As  my underlying ZnServer could spawn child threads to cope with concurrent POST requests:
	- in the context of a POST request treatment  a ChatMessageAnnoucement is made that communicates the message content to the ChatMessagesQueue announcer 
	- the actual writing operation of this new message to the ChatMessagesQueue is then realized in the context of the main server thread by the ChatMessagesQueue itself (that has subscribed to the ChatMessageAnnouncement at start)!

!ChatServer class methodsFor: 'public' stamp: 'FrancoisJarriges 6/3/2016 15:01'!
startOn: aPortNumber

	^ self new
		initializePort: aPortNumber;
		registerRoutes;
		registerErrorHandlers;
		start;
		yourself! !

!ChatServer class methodsFor: 'image-startup' stamp: 'FrancoisJarriges 6/15/2016 08:46'!
startup
	ChatServer stopAll.
	ChatServer startOn: 8080! !

!ChatServer class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 09:44'!
stopAll

	^ self allInstancesDo: #stop! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/15/2016 10:52'!
addMessage: aTeaRequest

	| newChatMessage |
	
	newChatMessage := (STONReader on: aTeaRequest entity string readStream) next.

	messagesQueue announcer 
		announce: (ChatMessageAnnouncement for: newChatMessage)
! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/5/2016 10:23'!
connectedUsers

	^ log users values 
		select: [ :eachUser | eachUser connected ]
		thenCollect: [ :eachConnectedUser | eachConnectedUser login ]
! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 22:13'!
initialize

	super initialize.
	messagesQueue := ChatMessagesQueue new.
	log := ChatUsersLogged  new
! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/3/2016 15:25'!
initializePort: anInteger

	teapotServer := Teapot configure: {
		#defaultOutput -> #text.
		#port -> anInteger.
		#debugMode -> true }
! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/4/2016 23:45'!
messageCount

	^ messagesQueue size! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/15/2016 16:00'!
messagesFrom: aTeaRequest

	| thisChatUserId | 

	thisChatUserId := aTeaRequest headers at: 'ChatUserId'.
	(log users at: thisChatUserId asSymbol) lifeSign: true.
	^ messagesQueue listFrom: (aTeaRequest at: #id)! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/15/2016 17:27'!
registerErrorHandlers
	"to be implemented in a different way"
	"teapotServer 
		exception: KeyNotFound -> (TeaResponse notFound body: 'No such message')"! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/15/2016 16:30'!
registerRoutes
	
	teapotServer 
		GET: '/users/connected' -> (Send message: #connectedUsers to: self); output: #ston;
		GET: '/messages/count' -> (Send message: #messageCount to: self); output: #ston;
		GET: '/messages/from/<id:IsInteger>' -> (Send message: #messagesFrom: to: self); output: #ston;
		PUT: '/messages/add' -> (Send message: #addMessage: to: self);
		PUT: 'users/login' -> (Send message: #userLogin: to: self);
		PUT: '/users/logout' -> (Send message: #userLogout: to: self)! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 23:49'!
start
		
	teapotServer start.
	log updateConnections 
	

! !

!ChatServer methodsFor: 'public' stamp: 'FrancoisJarriges 6/4/2016 23:50'!
stop
	
	teapotServer stop.
	log stopUpdateConnections.
	messagesQueue reset! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/15/2016 10:38'!
userLogin: aTeaRequest

	| loginChatMessage thisChatUserId |
	
	loginChatMessage := (STONReader on: aTeaRequest entity string readStream) next.
	thisChatUserId := aTeaRequest headers at: 'ChatUserId'.
	log users at: thisChatUserId asSymbol 
		ifAbsentPut: [ ChatUser withLogin: loginChatMessage sender].

	messagesQueue announcer 
		announce: (ChatMessageAnnouncement for: (ChatMessage 
			from: '-----'
			text: loginChatMessage sender, ' joined the chat room'))
! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/15/2016 16:57'!
userLogout: aTeaRequest

	| thisChatUserId thisUser |
	
	thisChatUserId := aTeaRequest headers at: 'ChatUserId'.
	thisUser := (log users at: thisChatUserId asSymbol)
		lifeSign: false;
		connected: false.
	messagesQueue announcer 
		announce: (ChatMessageAnnouncement for: (ChatMessage 
			from: '-----'
			text: thisUser login, ' exited from the chat room'))
! !

Object subclass: #ChatUser
	instanceVariableNames: 'login connected lifeSign'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-User'!

!ChatUser commentStamp: 'FrancoisJarriges 6/4/2016 17:31' prior: 0!
I represent a user in of this tiny chat application.
 
Internal Representation and Key Implementation Points.

    Instance Variables

	login:		<ByteString>


    Implementation Points!

!ChatUser class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/4/2016 22:06'!
withLogin: loginString
	
	^ self new 
		login: loginString;
		connected: true;
		lifeSign: true;
		yourself! !

!ChatUser methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 6/4/2016 22:03'!
connected
	^ connected ifNil: [ connected := false ]! !

!ChatUser methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 6/4/2016 22:03'!
connected: aBoolean
	connected := aBoolean! !

!ChatUser methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 6/4/2016 22:03'!
lifeSign
	^ lifeSign ifNil: [ lifeSign := false ]! !

!ChatUser methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 6/4/2016 22:04'!
lifeSign: aBoolean
	lifeSign := aBoolean! !

!ChatUser methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 07:29'!
login
	^ login ifNil: [ login := '' ]! !

!ChatUser methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 08:59'!
login: aByteString
	login := aByteString! !

!ChatUser methodsFor: 'printing' stamp: 'FrancoisJarriges 6/4/2016 17:31'!
printOn: aStream

	aStream 
		<< self login; 
		<< String crlf! !

Object subclass: #ChatUsersLogged
	instanceVariableNames: 'users checkConnections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatUsersLogged commentStamp: 'FrancoisJarriges 6/4/2016 22:09' prior: 0!
I represent all the users connections to the chat since the server started.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	users:				<Dictionary>
	checkConnections: 	<Boolean>


    Implementation Points
	!

!ChatUsersLogged methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/4/2016 22:21'!
initialize

	super initialize.
	users := Dictionary new.
	checkConnections := true! !

!ChatUsersLogged methodsFor: 'private' stamp: 'FrancoisJarriges 6/4/2016 23:50'!
stopUpdateConnections
	checkConnections := false! !

!ChatUsersLogged methodsFor: 'private' stamp: 'FrancoisJarriges 6/5/2016 11:20'!
updateConnections
	"Create and schedule a light process to check users connections live signs every 15 seconds."
	"When the server stops the process ends."
	
	[
		[ checkConnections ] whileTrue: [
			(Delay forSeconds: 15) wait.
			 users do: [ :eachUser | 
				eachUser connected: eachUser lifeSign.
				eachUser lifeSign: false ] ]
	] fork! !

!ChatUsersLogged methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/4/2016 10:17'!
users
	^ users! !

ChatClientPutMessage initialize!

ChatClientPutLogout initialize!

ChatClientGetLastMessageIndex initialize!

ChatClientPutLogin initialize!

ChatClientGetMessagesFromLastIndex initialize!

ChatClientGetConnectedUsers initialize!
